package popminimalexample;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
/**
 *
 * @author Behrang QasemiZadeh <zadeh at phil.hhu.de>
 */
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import java.util.HashMap;
import java.util.Set;

public class PoPWordNewFreqRandomNoOptim {

    
    static HashMap<String, SimpleSparseDouble> wordVectorMap;
    static SimpleSparseDouble sumUniversalVector;
    static int left = 5;
    static int right = 5;
    static int dimension = 666;
    private static HashMap<String, Integer> randomIndexInfo;
//    private static Set<String> targetVocab;
    static double filteredNumber;
    static double countAllSoFar = 0;

    static RandomGenerator rg;

    /**
     * Get index vector info
     *
     * @param word
     * @return
     */
    private static int getIndexVectorInfo(String word) {
        if (randomIndexInfo.containsKey(word)) {
            return randomIndexInfo.get(word);
        } else {
          
            int popIndex  = rg.getPositiveOnlyRandomVector();

            randomIndexInfo.put(word, popIndex);
            return popIndex;
        }
    }

    public static void main(String args[]) throws IOException, Exception {

        if(args.length!=1){
            System.out.println("Provide path to directory that contains your text files...");
        }
        rg = new RandomGenerator(dimension);
        System.out.println("Run basic PoP with java native libraries...");
        

        sumUniversalVector = new SimpleSparseDouble(dimension);
        wordVectorMap = new HashMap();
        randomIndexInfo = new HashMap();
        System.out.println("Context size is " + left + "+" + right);
        File dicFile = new File(args[0]);//args[0]);
   //     System.out.println("Reading text files from directory " + args[0]);
        for (File f : dicFile.listFiles()) {
            System.out.println("\t current file: " + f.getName());
            if (f.isFile()) {
                train(f.getCanonicalPath());
            }
        }

        System.out.println("Serlialising the obtained models to file (Both unweighted and weighted vectors are generated)...");
        serializeRawVectors(wordVectorMap, "pop-vectors-d" + dimension + "-cLR" + left + "-" + right);
        serializeRawVectorsPPMI(wordVectorMap, "pop-ppmi-vectors-d" + dimension + "-cLR" + left + "-" + right, 0);
    }

    public static void train(String file) throws FileNotFoundException, IOException, Exception {
        BufferedReader in = new BufferedReader(new FileReader(new File(file)));
        String line = null;
        while ((line = in.readLine()) != null) {
            String[] split = line.split("\\W+");
            for (int j = 0; j < split.length; j++) {
                String targetWord = split[j].trim();
//                if (targetVocab.contains(targetWord)) {
                    SimpleSparseDouble contextVector = null;
                    if (wordVectorMap.containsKey(targetWord)) {
                        contextVector = wordVectorMap.get(targetWord);
                    } else {
                        contextVector = new SimpleSparseDouble(dimension);
                        wordVectorMap.put(targetWord, contextVector);
                    }
                    contextVector.incFrequency();
                    for (int k = Math.max(0, j - left); k < j; k++) {
                        String contextWord = split[k];
                        addIndexVector(contextWord, contextVector);
                    }
                    for (int k = j + 1; k < Math.min(j + right, split.length); k++) {
                        String contextWord = split[k];
                        addIndexVector(contextWord, contextVector);
                    }
//                }

            }

        }

    }

    public static void addIndexVector(String indexWord, SimpleSparseDouble contextVector) {
        int indexInfo = getIndexVectorInfo(indexWord);
        int coordinate = rg.getDim(indexInfo);
        int value = rg.getValue(indexInfo);
        // if intersted, use the following weighting for some improvments, simply change the value to weight
        //double weight = 
        //-Math.log(value * 1.0 * sumAllValues / Math.pow(sumUniversalVector.getValue(coordinate), 2))
        contextVector.incValue(coordinate, value * 1.0);
    }

    private static void serializeRawVectors(HashMap<String, SimpleSparseDouble> wordVectorMap, String filename) throws IOException {
        File file = new File(filename);
        FileWriter writer = new FileWriter(file);
        BufferedWriter bfWriter = new BufferedWriter(writer, 2048576);
        for (String key : wordVectorMap.keySet()) {
            bfWriter.append(key);
            bfWriter.append("\t");
            bfWriter.append(wordVectorMap.get(key).toString());
            bfWriter.newLine();
        }
        bfWriter.close();
    }

    /**
     * Do the weighting and write the file
     * @param wordVectorMap
     * @param filename
     * @param min
     * @throws IOException 
     */
    private static void serializeRawVectorsPPMI(
            HashMap<String, SimpleSparseDouble> wordVectorMap,
            String filename, int min) throws IOException {

        SimpleSparseDouble sums = new SimpleSparseDouble(dimension);
        for (SimpleSparseDouble ssi : wordVectorMap.values()) {
            sums.addVector(ssi);
        }
        File file = new File(filename);
        FileWriter writer = new FileWriter(file);
        BufferedWriter bfWriter = new BufferedWriter(writer, 2048576);
        for (String key : wordVectorMap.keySet()) {
            if (wordVectorMap.get(key).getFrequency() > min) {
                bfWriter.append(key);
                bfWriter.append("\t");
                double[] ppmi = wordVectorMap.get(key).getPPMIDouble(sums);

                for (int i = 0; i < ppmi.length; i++) {
                    bfWriter.append(ppmi[i] + " ");
                }
                bfWriter.newLine();
            }
        }
        bfWriter.close();

    }

  

}
