package popminimalexample;

import java.util.Random;

/**
 *
 * @author Behrang QasemiZadeh <zadeh at phil.hhu.de>
 * PoP random vector generator: note that using random vectors with more than
 * <b>one</b> non-zero elements is also possible. However, if using random index
 * vectors with more than one-zero elements is not desirable due to the imposed
 * computational complexity.
 */
public class RandomGenerator {

    int reducedDimension;
    Random randomElement;
    Random randomValuePositive;

    public RandomGenerator(int reducedDimension) {
        this.reducedDimension = reducedDimension;
        randomElement = new Random();
        randomValuePositive = new Random();
    }

    /**
     * Get the next element of index vectors that will be non-zero randomly
     *
     * @return
     */
    private short getNextRandomIndexElement() {
        short l = (short) randomElement.nextInt(reducedDimension);
        return l;
    }

    /**
     * Get atomic Positive-Only-Random Vector packed in 32bits
     *
     * @return
     */
    public int getPositiveOnlyRandomVector() {
        short val = (short) Math.floor(1.0
                / Math.pow(
                        randomValuePositive.nextDouble(), .7
                )
        );
        short index = (short) getNextRandomIndexElement();
        int l = ((index) << 16) | (val & 0xffff);

        return l;

    }

    public int[] getPoPIndex() {
        int index = (short) randomElement.nextInt(reducedDimension);
        int val = (int) Math.floor(1.0
                / Math.pow(
                        randomValuePositive.nextDouble(), .7
                )
        );
        return new int[]{index, val};
    }

    /**
     * Get dimensionality from packed data
     * @param packedData
     * @return 
     */
    public int getDim(int packedData) {
        return packedData >> 16;
    }

    /**
     * Get value from packed data
     * @param packedData
     * @return 
     */
    public int getValue(int packedData) {
        return packedData & 0xFFFF;
    }

    public static void main(String[] test) {

        System.out.println("Example of generating a random vector....");
        
        System.out.println("===");
        System.out.println("Get one 3000 dimensional vector");
        RandomGenerator rg = new RandomGenerator(3000);
        int d = rg.getPositiveOnlyRandomVector();

        System.out.println("\tNonzero coordinate is: " + rg.getDim(d));
        System.out.println("\tValue for this coordinate is " + rg.getValue(d));
        
        System.out.println("===");
        System.out.println("Get one 300 dimensional vector");
        RandomGenerator rg2 = new RandomGenerator(300);
        int d2 = rg2.getPositiveOnlyRandomVector();

        System.out.println("\tNonzero coordinate is: " + rg.getDim(d2));
        System.out.println("\tValue for this coordinate is " + rg.getValue(d2));

    }

}
