/*
 * Copyright (C) 2016 Behrang QasemiZadeh <zadeh at phil.hhu.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package popminimalexample;


import java.util.HashMap;
import java.util.Set;

/**
 *
 * @author Behrang QasemiZadeh <zadeh at phil.hhu.de>
 */
public class SimpleSparseDouble {

    public static final String LABEL_DELIMIT = "\t\t";
    public static final String VECTOR_IND_VLU_DELIMIT = ":";
    public static final String DIM_DELIMITER = " ";
    private HashMap<Integer,Double> vector;

    private double frequency;
    private int dimensionality;
    double[] dense;

    

    public SimpleSparseDouble(double[] vec) {
        vector = new HashMap();
       
        this.dimensionality = vec.length;
        for (int i = 0; i < vec.length; i++) {
            vector.put(i, vec[i]);

        }
    }

    public SimpleSparseDouble(int dimensionality) {
        vector = new HashMap();
       
        this.dimensionality = dimensionality;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        Set<Integer> keys = vector.keySet();
        for (int key : keys) {
            sb.append(key).append(VECTOR_IND_VLU_DELIMIT).append(vector.get(key)).append(DIM_DELIMITER);
        }
        return sb.toString();
    }

    public void incFrequency() {
        this.frequency++;
    }

    public double getFrequency() {
        return frequency;
    }

    public void addVector(SimpleSparseDouble v) {
        for (int i : v.getIndices()) {
            double elementValue = this.getValue(i) + (v.getValue(i));
            this.vector.put(i, elementValue);
        }
       
    }

    public static SimpleSparseDouble fromStringSparse(String line, int dim) {
        //    System.out.println(line);
        SimpleSparseDouble sv = new SimpleSparseDouble(dim);
        String[] splitLine = line.split(DIM_DELIMITER);
        for (int i = 0; i < splitLine.length; i++) {
            String[] splitBit = splitLine[i].split(VECTOR_IND_VLU_DELIMIT);
            int index = Integer.parseInt(splitBit[0]);
            double value = Double.parseDouble(splitBit[1]);
            sv.setValue(index, value);
        }
       
        return sv;

    }

    public static double[] fromStringDense(String line, int dim) throws Exception {
        double[] dense = new double[dim];
        String[] splitLine = line.split(DIM_DELIMITER);
        for (int i = 0; i < dim; i++) {
            dense[i] = Double.parseDouble(splitLine[i]);
            if (!Double.isFinite(dense[i])) {
                throw new Exception("There is some error in the input file at dim " + i + " in line ");
            }
        }
        return dense;
    }

    public void convertToPPMIDouble(SimpleSparseDouble ssAll) {
        double sumAll = ssAll.getSum();
        double sumThisRow = this.getSum();
        double pmiVec;
        for (int idx = 0; idx < this.dimensionality; idx++) {
            double pmi = Math.log((getValue(idx) * sumAll) / (sumThisRow * ssAll.getValue(idx)));
            pmiVec = Double.max(0, pmi);
            if (!Double.isFinite(pmiVec)) {
                if (Double.isInfinite(pmiVec)) {
                    pmiVec = 0xFFFF;
                } else {
                    pmiVec = 0;
                }
            }
            this.vector.put(idx, pmiVec);
        }

    }

    public double[] getPPMIDouble(SimpleSparseDouble ssAll) {
        double sumAll = ssAll.getSum();
        double sumThisRow = this.getSum();
        double[] pmiVec = new double[this.dimensionality];
        for (int idx = 0; idx < this.dimensionality; idx++) {
            double pmi = Math.log((getValue(idx) * sumAll) / (sumThisRow * ssAll.getValue(idx)));
            pmiVec[idx] = Double.max(0, pmi);
            if (!Double.isFinite(pmiVec[idx])) {
                if (Double.isInfinite(pmiVec[idx])) {
                    pmiVec[idx] = 0xFFFF;
                } else {
                    pmiVec[idx] = 0;
                }
            }
        }
        return pmiVec;
    }

    public void setValue(int index, double value) {
        vector.put(index, value);
      
    }

    public void incValue(int index, double value) {
        double newValue = this.getValue(index) + value;
        vector.put(index, newValue);
      
    }

    public double getValue(int index) {
        if (vector.containsKey(index)) {
            double get = vector.get(index);
            return get;
        } else {
            return 0;
        }

    }

    public Set<Integer> getIndices() {
        return vector.keySet();
    }

    public double getSum() {
        double sum = 0;
        for (double value : vector.values()) {
            sum += value;
        }
        return sum;
    }

}
